package com.sun.org.glassfish.external.statistics.impl;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Map;
import java.lang.reflect.Proxy;
import java.lang.reflect.InvocationHandler;
import com.sun.org.glassfish.external.statistics.AverageRangeStatistic;

public final class AverageRangeStatisticImpl extends StatisticImpl implements AverageRangeStatistic, InvocationHandler
{
    private long currentVal;
    private long highWaterMark;
    private long lowWaterMark;
    private long numberOfSamples;
    private long runningTotal;
    private final long initCurrentVal;
    private final long initHighWaterMark;
    private final long initLowWaterMark;
    private final long initNumberOfSamples;
    private final long initRunningTotal;
    private final AverageRangeStatistic as;
    
    public AverageRangeStatisticImpl(long curVal, long highMark, long lowMark, String name, String unit, String desc, long startTime, long sampleTime) {
        super(name, unit, desc, startTime, sampleTime);
        this.currentVal = 0L;
        this.highWaterMark = Long.MIN_VALUE;
        this.lowWaterMark = Long.MAX_VALUE;
        this.numberOfSamples = 0L;
        this.runningTotal = 0L;
        this.as = (AverageRangeStatistic)Proxy.newProxyInstance(AverageRangeStatistic.class.getClassLoader(), new Class[] { AverageRangeStatistic.class }, this);
        this.currentVal = curVal;
        this.initCurrentVal = curVal;
        this.highWaterMark = highMark;
        this.initHighWaterMark = highMark;
        this.lowWaterMark = lowMark;
        this.initLowWaterMark = lowMark;
        this.numberOfSamples = 0L;
        this.initNumberOfSamples = this.numberOfSamples;
        this.runningTotal = 0L;
        this.initRunningTotal = this.runningTotal;
    }
    
    public synchronized AverageRangeStatistic getStatistic() {
        return this.as;
    }
    
    @Override
    public synchronized String toString() {
        return super.toString() + AverageRangeStatisticImpl.NEWLINE + "Current: " + this.getCurrent() + AverageRangeStatisticImpl.NEWLINE + "LowWaterMark: " + this.getLowWaterMark() + AverageRangeStatisticImpl.NEWLINE + "HighWaterMark: " + this.getHighWaterMark() + AverageRangeStatisticImpl.NEWLINE + "Average:" + this.getAverage();
    }
    
    @Override
    public synchronized Map getStaticAsMap() {
        Map<String, Long> m = super.getStaticAsMap();
        m.put("current", Long.valueOf(this.getCurrent()));
        m.put("lowwatermark", Long.valueOf(this.getLowWaterMark()));
        m.put("highwatermark", Long.valueOf(this.getHighWaterMark()));
        m.put("average", Long.valueOf(this.getAverage()));
        return m;
    }
    
    @Override
    public synchronized void reset() {
        super.reset();
        this.currentVal = this.initCurrentVal;
        this.highWaterMark = this.initHighWaterMark;
        this.lowWaterMark = this.initLowWaterMark;
        this.numberOfSamples = this.initNumberOfSamples;
        this.runningTotal = this.initRunningTotal;
        this.sampleTime = -1L;
    }
    
    public synchronized long getAverage() {
        if (this.numberOfSamples == 0L) {
            return -1L;
        }
        return this.runningTotal / this.numberOfSamples;
    }
    
    public synchronized long getCurrent() {
        return this.currentVal;
    }
    
    public synchronized void setCurrent(long curVal) {
        this.currentVal = curVal;
        this.lowWaterMark = ((curVal >= this.lowWaterMark) ? this.lowWaterMark : curVal);
        this.highWaterMark = ((curVal >= this.highWaterMark) ? curVal : this.highWaterMark);
        ++this.numberOfSamples;
        this.runningTotal += curVal;
        this.sampleTime = System.currentTimeMillis();
    }
    
    public synchronized long getHighWaterMark() {
        return this.highWaterMark;
    }
    
    public synchronized long getLowWaterMark() {
        return this.lowWaterMark;
    }
    
    public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
        this.checkMethod(method);
        Object result;
        try {
            result = method.invoke(this, args);
        }
        catch (InvocationTargetException e) {
            throw e.getTargetException();
        }
        catch (Exception e2) {
            throw new RuntimeException("unexpected invocation exception: " + e2.getMessage());
        }
        return result;
    }
}
